// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) 2008-2011 ZenJu (zhnmju123 AT gmx.de)                    *
// **************************************************************************

#ifndef SHADOWCOPY_H
#define SHADOWCOPY_H

#ifdef SHADOWDLL_EXPORTS
#define SHADOWDLL_API extern "C" __declspec(dllexport)
#else
#define SHADOWDLL_API extern "C" __declspec(dllimport)
#endif

#include <zen/build_info.h>
#include <zen/win_ver.h>


class IVssBackupComponents;

namespace shadow
{
/*--------------
  |declarations|
  --------------*/

//COM needs to be initialized before calling any of these functions! CoInitializeEx/CoUninitialize
struct ShadowData;
typedef ShadowData* ShadowHandle;

//volumeName *must* end with "\"
SHADOWDLL_API
ShadowHandle createShadowCopy(const wchar_t* volumeName,      // in  Returns nullptr on failure!
                              wchar_t*		 errorMessage,    // out
                              unsigned int	 errorBufferLen); // in

//don't forget to release the backupHandle after shadow copy is not needed anymore!
SHADOWDLL_API
void releaseShadowCopy(ShadowHandle handle);

SHADOWDLL_API
void getShadowVolume(ShadowHandle handle, //shadowVolName does *not* end with "\"
                     wchar_t*	  buffer,
                     unsigned int bufferLen);

//##########################################################################################


/*----------
  |typedefs|
  ----------*/
typedef ShadowHandle (*CreateShadowCopyFct)(const wchar_t* volumeName,
                                            wchar_t*	    errorMessage,
                                            unsigned int   errorBufferLen);

typedef void (*ReleaseShadowCopyFct)(ShadowHandle handle);

typedef void (*GetShadowVolumeFct)(ShadowHandle handle,
                                   wchar_t*	   buffer,
                                   unsigned int bufferLen);


/*--------------
  |symbol names|
  --------------*/
//(use const pointers to ensure internal linkage)
const char createShadowCopyFctName [] = "createShadowCopy";
const char releaseShadowCopyFctName[] = "releaseShadowCopy";
const char getShadowVolumeFctName  [] = "getShadowVolume";

/*---------------
  |library names|
  ---------------*/

inline
const wchar_t* getDllName()
{
    // distinguish a bunch of VSS builds: we use XP, Server 2003 and Server 2008 R2 implementations...
    // VSS version and compatibility overview: http://msdn.microsoft.com/en-us/library/aa384627(VS.85).aspx

    if (zen::win7OrLater()) //Windows Server 2008 R2 or Windows 7
        return zen::is64BitBuild ?
               L"Shadow_Windows7_x64.dll" :
               L"Shadow_Windows7_Win32.dll";
    //else if (vistaOrLater()) -> skip Windows Server 2008 and Windows Vista
    //			   ;
    else if (zen::winServer2003orLater()) //Windows Server 2003 and Windows Server 2003 R2
        return zen::is64BitBuild ?
               L"Shadow_Server2003_x64.dll" :
               L"Shadow_Server2003_Win32.dll";
    else //Windows XP
        return zen::is64BitBuild ?
               L"Shadow_XP_x64.dll" :
               L"Shadow_XP_Win32.dll";
}
}

#endif //SHADOWCOPY_H
