// **************************************************************************
// * This file is part of the zenXML project. It is distributed under the   *
// * Boost Software License, Version 1.0. See accompanying file             *
// * LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt.       *
// * Copyright (C) 2011 ZenJu (zhnmju123 AT gmx.de)                         *
// **************************************************************************

#include <set>
#include <map>
#include <deque>
#include <vector>
#include <list>
#include <utility>
#include "xml.h"

using namespace zen;

namespace
{
class Dummy {};

void unit_test()
{
    //compile time checks only

    assert_static(!IsStlContainer<wchar_t>                     ::result);
    assert_static(!IsStlContainer<char>                        ::result);
    assert_static(!IsStlContainer<Dummy>                       ::result);
    assert_static(!IsStlContainer<EmptyType>                   ::result);
    assert_static(IsStlContainer<std::set      <int>>          ::result);
    assert_static(IsStlContainer<std::deque    <float>>        ::result);
    assert_static(IsStlContainer<std::list     <size_t>>       ::result);
    assert_static((IsStlContainer<std::map     <double, char>> ::result));
    assert_static((IsStlContainer<std::multimap<short, double>>::result));
    assert_static(IsStlContainer <std::vector  <wchar_t>>      ::result);
    assert_static((IsStlPair     <std::pair<int, double>>      ::result));
    assert_static(!IsStlPair<Dummy>                            ::result);

    assert_static(!IsStringLike<Dummy>::result);
    assert_static(!IsStringLike<int>::result);
    assert_static(!IsStringLike<double>::result);
    assert_static(!IsStringLike<short>::result);
    assert_static(IsStringLike<char>::result);
    assert_static(IsStringLike<const wchar_t>::result);
    assert_static(IsStringLike<const char>::result);
    assert_static(IsStringLike<wchar_t>::result);
    assert_static(IsStringLike<char*>::result);
    assert_static(IsStringLike<wchar_t*>::result);
    assert_static(IsStringLike<char* const>::result);
    assert_static(IsStringLike<wchar_t* const>::result);
    assert_static(IsStringLike<const char*>::result);
    assert_static(IsStringLike<const char* const>::result);
    assert_static(IsStringLike<const wchar_t*>::result);
    assert_static(IsStringLike<const wchar_t* const>::result);
    assert_static(IsStringLike<const char[4]>::result);
    assert_static(IsStringLike<const wchar_t[4]>::result);
    assert_static(IsStringLike<char[4]>::result);
    assert_static(IsStringLike<wchar_t[4]>::result);
    assert_static(IsStringLike<std::string>::result);
    assert_static(IsStringLike<std::wstring>::result);
    assert_static(IsStringLike<const std::string>::result);
    assert_static(IsStringLike<const std::wstring>::result);
    assert_static(IsStringLike<const std::string&>::result);
    assert_static(IsStringLike<const std::wstring&>::result);
    assert_static(IsStringLike<std::string&>::result);
    assert_static(IsStringLike<std::wstring&>::result);

    assert_static(!(IsSameType<GetCharType<int>::Result, char>::result));
    assert_static(!(IsSameType<GetCharType<double>::Result, char>::result));
    assert_static(!(IsSameType<GetCharType<short>::Result, char>::result));
    assert_static((IsSameType<GetCharType<char>::Result, char>::result));
    assert_static((IsSameType<GetCharType<wchar_t>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<const char>::Result, char>::result));
    assert_static((IsSameType<GetCharType<const wchar_t>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<char*>::Result, char>::result));
    assert_static((IsSameType<GetCharType<wchar_t*>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<char* const>::Result, char>::result));
    assert_static((IsSameType<GetCharType<wchar_t* const>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<const char*>::Result, char>::result));
    assert_static((IsSameType<GetCharType<const char* const>::Result, char>::result));
    assert_static((IsSameType<GetCharType<const wchar_t*>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<const wchar_t* const>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<const char[4]>::Result, char>::result));
    assert_static((IsSameType<GetCharType<const wchar_t[4]>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<char[4]>::Result, char>::result));
    assert_static((IsSameType<GetCharType<wchar_t[4]>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<std::string>::Result, char>::result));
    assert_static((IsSameType<GetCharType<std::wstring>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<const std::string>::Result, char>::result));
    assert_static((IsSameType<GetCharType<const std::wstring>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<const std::string&>::Result, char>::result));
    assert_static((IsSameType<GetCharType<const std::wstring&>::Result, wchar_t>::result));
    assert_static((IsSameType<GetCharType<std::string&>::Result, char>::result));
    assert_static((IsSameType<GetCharType<std::wstring&>::Result, wchar_t>::result));
}
}
